#! /bin/sh
#=======================================================================
# Run validation tests on bibXXX (XXX = lex, parse, or unlex).
#
# Usage:
#	./docheck.sh PROGRAM test1 test2 ...
#
# The first argument is the name of the program to test; it should be
# absolute, relative, or else known to be in the search path.
#
# The second through last of the files are the basenames of test
# files.
#
# [07-Oct-2015] Allow SED value to be supplied from the environment
#               because of different behavior of sed on OpenBSD with
#		'\r' pattern to match a carriage return.
# [03-Jun-1998]
#=======================================================================

# Validate the argument count
if test $# -lt 2
then
    echo Usage: ./docheck.sh PROGRAM test1 test2 ...
    exit 1
fi

# Validate the program name
PROGRAM=$1
if test -f $PROGRAM
then
    true
else
    PROGRAM=`which $PROGRAM`
fi

if test -f $PROGRAM
then
    true
else
    echo $1 not found
    exit 1
fi

# Discard the first argument; what remains are the test basenames
shift

CMP=cmp
DIFF="diff -w -b"
GREP=grep
HEAD="head -40"
RM="rm -f"
SED=${SED-sed}
VM=${VM}

test -f ERROR-COUNT || echo 0 > ERROR-COUNT
test -f TEST-COUNT  || echo 0 > TEST-COUNT

for f in "$@"
do
    if test -f $f.opt
    then
	echo $f: options `cat $f.opt`
	## echo "DEBUG: ${VM} ${PROGRAM} `cat $f.opt` $f.in 2>$f.err"
	${VM} ${PROGRAM} `cat $f.opt` $f.in 2>$f.err | ${GREP} -v '^yydebug:' | ${SED} -e 's/\r//g' >$f.out
    else
	echo $f
	## echo "DEBUG: ${VM} ${PROGRAM} $f.in 2>$f.err"
	${VM} ${PROGRAM}              $f.in 2>$f.err | ${GREP} -v '^yydebug:' | ${SED} -e 's/\r//g' >$f.out
    fi

    n=`cat TEST-COUNT`
    echo `expr $n + 2` > TEST-COUNT

    # The sed step is necessary to handle SGI IRIX 6 lex output, and
    # bison output, and is harmless on other systems.
    ${SED} -e 's/Syntax error/syntax error/'    \
	   -e 's/parse error/syntax error/'     \
	   -e '/wine: Module not found/d'       \
	   -e 's/\r//g'                         \
	   < $f.err >$f.er2
    mv $f.er2 $f.err

    # [04-Jul-2014] the okay/*.err.2 files represent alternate
    # valid outputs from different lex/flex versions: we accept
    # both pairs
    OKAYERR=okay/$f.err
    test -f ${OKAYERR}.2 && ${CMP} -s ${OKAYERR}.2 $f.err && OKAYERR=${OKAYERR}.2
    test -f ${OKAYERR}.3 && ${CMP} -s ${OKAYERR}.3 $f.err && OKAYERR=${OKAYERR}.3

    if ${CMP} ${OKAYERR} $f.err
    then ${RM} $f.err
    else
	echo Failure on $f.err ...
	${DIFF} ${OKAYERR} $f.err | ${HEAD}
	n=`cat ERROR-COUNT`
	echo `expr $n + 1` > ERROR-COUNT
    fi

    OKAYOUT=okay/$f.out
    test -f ${OKAYOUT}.2 && ${CMP} -s ${OKAYOUT}.2 $f.out && OKAYOUT=${OKAYOUT}.2
    test -f ${OKAYOUT}.3 && ${CMP} -s ${OKAYOUT}.3 $f.out && OKAYOUT=${OKAYOUT}.3

    if ${CMP} ${OKAYOUT} $f.out
    then ${RM} $f.out
    else
	echo Failure on $f.out ...
	${DIFF} ${OKAYOUT} $f.out | ${HEAD}
	n=`cat ERROR-COUNT`
	echo `expr $n + 1` > ERROR-COUNT
    fi
done
